@@ -291,8 +291,8 @@ class IOBase(object):
 
     __slots__ = "__bit_ioctl_off", "__bit_ioctl_on", "_bitaddress", \
                 "_bitshift", "_bitlength", "_byteorder", "_defaultvalue", \
-                "_iotype", "_length", "_name", "_parentdevice", \
-                "_read_only_io", "_signed", "_slc_address", "bmk", "export"
+                "_export", "_iotype", "_length", "_name", "_parentdevice", \
+                "_read_only_io", "_signed", "_slc_address", "bmk"
 
     def __init__(self, parentdevice, valuelist: list, iotype: int, byteorder: str, signed: bool):
         """
@@ -325,7 +325,7 @@ class IOBase(object):
         self._name = valuelist[0]
         self._signed = signed
         self.bmk = valuelist[6]
-        self.export = bool(valuelist[4])
+        self._export = int(valuelist[4]) & 1
 
         int_startaddress = int(valuelist[3])
         if self._bitshift:
@@ -514,6 +514,10 @@ class IOBase(object):
         """
         return self._byteorder
 
+    def _get_export(self) -> bool:
+        """Return value of export flag."""
+        return bool(self._export & 1)
+
     def _get_iotype(self) -> int:
         """
         Gibt io type zurueck.
@@ -522,6 +526,12 @@ class IOBase(object):
         """
         return self._iotype
 
+    def _set_export(self, value: bool) -> None:
+        """Set value of export flag and remember this change for export."""
+        if type(value) != bool:
+            raise ValueError("Value must be <class 'bool'>")
+        self._export = 2 + int(value)
+
     def get_defaultvalue(self):
         """
         Gibt die Defaultvalue von piCtory zurueck.
@@ -614,7 +624,7 @@ class IOBase(object):
             # Versuchen egal welchen Typ in Bool zu konvertieren
             value = bool(value)
 
-            if self._parentdevice._modio._direct_output:
+            if self._parentdevice._shared_procimg:
                 # Direktes Schreiben der Outputs
 
                 if self._parentdevice._modio._run_on_pi:
@@ -630,6 +640,7 @@ class IOBase(object):
                             )
                         except Exception as e:
                             self._parentdevice._modio._gotioerror("ioset", e)
+                            return
 
                 elif hasattr(self._parentdevice._modio._myfh, "ioctl"):
                     # IOCTL über Netzwerk
@@ -643,6 +654,7 @@ class IOBase(object):
                         except Exception as e:
                             self._parentdevice._modio._gotioerror(
                                 "net_ioset", e)
+                            return
 
                 else:
                     # IOCTL in Datei simulieren
@@ -655,28 +667,26 @@ class IOBase(object):
                         )
                     except Exception as e:
                         self._parentdevice._modio._gotioerror("file_ioset", e)
+                        return
 
-            else:
-                # Gepuffertes Schreiben der Outputs
+            # Für Bitoperationen sperren
+            self._parentdevice._filelock.acquire()
 
-                # Für Bitoperationen sperren
-                self._parentdevice._filelock.acquire()
+            # Hier gibt es immer nur ein byte, als int holen
+            int_byte = self._parentdevice._ba_devdata[self._slc_address.start]
 
-                # Hier gibt es immer nur ein byte, als int holen
-                int_byte = self._parentdevice._ba_devdata[self._slc_address.start]
+            # Aktuellen Wert vergleichen und ggf. setzen
+            if not bool(int_byte & self._bitshift) == value:
+                if value:
+                    int_byte += self._bitshift
+                else:
+                    int_byte -= self._bitshift
 
-                # Aktuellen Wert vergleichen und ggf. setzen
-                if not bool(int_byte & self._bitshift) == value:
-                    if value:
-                        int_byte += self._bitshift
-                    else:
-                        int_byte -= self._bitshift
+                # Zurückschreiben wenn verändert
+                self._parentdevice._ba_devdata[self._slc_address.start] = \
+                    int_byte
 
-                    # Zurückschreiben wenn verändert
-                    self._parentdevice._ba_devdata[self._slc_address.start] = \
-                        int_byte
-
-                self._parentdevice._filelock.release()
+            self._parentdevice._filelock.release()
 
         else:
             if type(value) != bytes:
@@ -694,7 +704,7 @@ class IOBase(object):
                     )
                 )
 
-            if self._parentdevice._modio._direct_output:
+            if self._parentdevice._shared_procimg:
                 with self._parentdevice._modio._myfh_lck:
                     try:
                         self._parentdevice._modio._myfh.seek(
@@ -705,8 +715,9 @@ class IOBase(object):
                             self._parentdevice._modio._myfh.flush()
                     except IOError as e:
                         self._parentdevice._modio._gotioerror("ioset", e)
-            else:
-                self._parentdevice._ba_devdata[self._slc_address] = value
+                        return
+
+            self._parentdevice._ba_devdata[self._slc_address] = value
 
     def unreg_event(self, func=None, edge=None) -> None:
         """
@@ -850,6 +861,7 @@ class IOBase(object):
     address = property(_get_address)
     byteorder = property(_get_byteorder)
     defaultvalue = property(get_defaultvalue)
+    export = property(_get_export, _set_export)
     length = property(__len__)
     name = property(__str__)
     type = property(_get_iotype)
@@ -1217,6 +1229,13 @@ class StructIO(IOBase):
             frm == frm.lower()
         )
         self.__frm = bofrm + frm
+        if "export" in kwargs:
+            # Use export property to remember given value for export
+            self.export = kwargs["export"]
+        else:
+            # User could change parent IO settings before replace to force
+            # export, so use parent settings for the new IO
+            self._export = parentio._export
 
         # Platz für neuen IO prüfen
         if not (self._slc_address.start >=
